--[[
Interface: 1.7.1.0 b10490

Copyright (C) GtX (Andy), 2020

Author: GtX | Andy
Date: 19.11.2020

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No changes are to be made to this script without permission from GtX | Andy

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
An diesem Skript dürfen ohne Genehmigung von GtX | Andy keine Änderungen vorgenommen werden
]]


local buildId = 1
local versionString = "0.0.0.0"

local modName = g_currentModName
local modDirectory = g_currentModDirectory

local function addFarmDetailElements(rootElement)
    if rootElement ~= nil then
        local lastFocusedElementName
        local lastFocusedElement = FocusManager:getFocusedElement()

        if lastFocusedElement ~= nil and lastFocusedElement.name ~= nil then
            lastFocusedElementName = lastFocusedElement.name
        end

        FocusManager:setGui(rootElement.name)

        function onPickColor(colourIndex, element)
            if colourIndex ~= nil then
                local farm = g_farmManager:getFarmById(g_currentMission.player.farmId)

                if farm ~= nil and element ~= nil then
                    if colourIndex ~= farm.color then
                        farm.color = colourIndex

                        g_messageCenter:publish(MessageType.FARM_PROPERTY_CHANGED, {
                            farm.farmId
                        })
                    end

                    element.farmColorPreview:setImageColor(nil, unpack(Farm.COLORS[farm.color]))
                    element.farmColor = farm.color
                end
            end
        end

        local farmColorElement = BitmapElement:new(rootElement)
        farmColorElement:applyProfile("shopColorButtonPicked")
        farmColorElement:updatePositionForOrigin("middleCenter")
        farmColorElement:updateScreenAlign("middleCenter")
        farmColorElement.size = GuiUtils.getNormalizedValues("300px 32px", farmColorElement.outputSize, farmColorElement.size)

        local farmColorTitleElement = TextElement:new(rootElement)
        farmColorTitleElement:applyProfile("multiTextOptionSettingsTitle")
        farmColorTitleElement:updateScreenAlign("topLeft")
        farmColorTitleElement:setText(rootElement.l10n:getText("ui_farmColor"))

        local farmColorButtonElement = ButtonElement:new(rootElement)
        farmColorButtonElement:applyProfile("shopColorButton")
        farmColorButtonElement.screenAlign = rootElement.textSavegameName.screenAlign
        farmColorButtonElement.positionOrigin = rootElement.textSavegameName.positionOrigin
        farmColorButtonElement.margin = rootElement.textSavegameName.margin
        farmColorButtonElement.size = rootElement.textSavegameName.size
        farmColorButtonElement.position = rootElement.textSavegameName.position

        farmColorButtonElement.onClickCallback = function(element)
            local farm = g_farmManager:getFarmById(g_currentMission.player.farmId)

            if farm ~= nil then
                if Farm.NAMED_COLORS == nil then
                    Farm.NAMED_COLORS = {
                        {name = element.l10n:getText("farmColour_green", modName), color = {0.25, 1, 0.25, 1}},
                        {name = element.l10n:getText("farmColour_darkBlue", modName), color = {0, 0.0446, 0.187, 1}},
                        {name = element.l10n:getText("farmColour_yellow", modName), color = {0.9386, 0.4678, 0.0123, 1}},
                        {name = element.l10n:getText("farmColour_orange", modName), color = {0.8832, 0.1636, 0.0046, 1}},
                        {name = element.l10n:getText("farmColour_red", modName), color = {0.5732, 0.005, 0.005, 1}},
                        {name = element.l10n:getText("farmColour_lightBlue", modName), color = {0, 0.2348, 0.7969, 1}},
                        {name = element.l10n:getText("farmColour_pink", modName), color = {0.8879, 0.0545, 0.3005, 1}},
                        {name = element.l10n:getText("farmColour_purple", modName), color = {0.0908, 0.004, 0.1301, 1}}
                    }

                    for i = 1, #Farm.COLORS do
                        local r, g, b, a = unpack(Farm.COLORS[i])

                        if Farm.NAMED_COLORS[i] ~= nil then
                            local nr, ng, nb, na = unpack(Farm.NAMED_COLORS[i].color)

                            -- If another mod has not changed the colour then keep the names
                            if math.abs(r - nr) > 0.1 or math.abs(g - ng) > 0.1 or math.abs(b - nb) > 0.1 then
                                Farm.NAMED_COLORS[i].name = ""
                                Farm.NAMED_COLORS[i].color = {r, g, b, a}
                            end
                        else
                            Farm.NAMED_COLORS[i] = {
                                name = "",
                                color = {r, g, b, a}
                            }
                        end
                    end
                end

                g_inputBinding:setShowMouseCursor(true)

                g_gui:showColorPickerDialog({
                    colors = Farm.NAMED_COLORS,
                    defaultColor = Farm.NAMED_COLORS[farm.color].color,
                    callback = onPickColor,
                    args = element
                })
            end
        end

        GuiOverlay.createOverlay(farmColorButtonElement.overlay)

        farmColorButtonElement:addElement(farmColorElement)
        farmColorButtonElement:addElement(farmColorTitleElement)

        local textFarmNameTitleElement = TextElement:new(rootElement)
        textFarmNameTitleElement:applyProfile("multiTextOptionSettingsTitle")
        textFarmNameTitleElement:updateScreenAlign("topLeft")
        textFarmNameTitleElement:setText(rootElement.l10n:getText("ui_farmName"))

        local textFarmNameElement = TextInputElement:new(rootElement)
        textFarmNameElement:applyProfile("ingameMenuGameSettingsTextInput")
        textFarmNameElement.text = rootElement.l10n:getText("ui_defaultFarmName")

        textFarmNameElement.onEnterPressedCallback = function(element)
            local farmName = element.textFarmName.text

            if farmName ~= element.textFarmName then
                if farmName ~= "" then
                    local filteredName = filterText(farmName, true, true)

                    if filteredName ~= farmName then
                        farmName = filteredName
                        g_logManager:info("The farm name you entered contains profanity and has been adjusted!")
                    end
                else
                    farmName = element.l10n:getText("ui_defaultFarmName")
                end

                element.textFarmName:setText(farmName)
                element.farmName = farmName

                local farm = g_farmManager:getFarmById(g_currentMission.player.farmId)

                if farm ~= nil then
                    farm.name = farmName
                    g_messageCenter:publish(MessageType.FARM_PROPERTY_CHANGED, {
                        farm.farmId
                    })
                end
            end
        end

        textFarmNameElement.onEscPressedCallback = function(element)
            element.textFarmName:setText(element.farmName or "")
        end

        GuiOverlay.createOverlay(textFarmNameElement.overlay)

        textFarmNameElement:addElement(textFarmNameTitleElement)

        rootElement.boxLayout:addElement(textFarmNameElement)
        rootElement.boxLayout:addElement(farmColorButtonElement)

        rootElement.textFarmName = textFarmNameElement
        rootElement.farmColorPreview = farmColorElement

        rootElement.boxLayout:updateAbsolutePosition()

        FocusManager:loadElementFromCustomValues(textFarmNameElement)
        FocusManager:linkElements(textFarmNameElement, FocusManager.TOP, rootElement.multiAutoSaveInterval)
        FocusManager:linkElements(textFarmNameElement, FocusManager.BOTTOM, farmColorButtonElement)

        FocusManager:loadElementFromCustomValues(farmColorButtonElement)
        FocusManager:linkElements(farmColorButtonElement, FocusManager.TOP, textFarmNameElement)
        FocusManager:linkElements(farmColorButtonElement, FocusManager.BOTTOM, rootElement.buttonPauseGame)

        FocusManager:linkElements(rootElement.buttonPauseGame, FocusManager.TOP, farmColorButtonElement)

        if lastFocusedElementName ~= nil then
            FocusManager:setGui(lastFocusedElementName)
        end

        return true
    end

    return false
end

local function resetToSingleplayer(farm, superFunc)
    local saveGameColour = farm.color
    superFunc(farm)
    farm.color = saveGameColour
end

local function updateGameSettings(frame)
    local farm = g_farmManager:getFarmById(g_currentMission.player.farmId)

    if farm ~= nil then
        if frame.textFarmName ~= nil then
            frame.textFarmName:setText(farm.name)
            frame.textFarmName:setDisabled(false)

            frame.farmName = farm.name
        end

        if frame.farmColorPreview ~= nil then
            frame.farmColorPreview:setImageColor(nil, unpack(Farm.COLORS[farm.color or 1]))
            frame.farmColor = farm.color
        end
    else
        frame.farmName = frame.l10n:getText("ui_defaultFarmName")
    end
end

local function loadMission(mission)
    if not mission.missionDynamicInfo.isMultiplayer then
        if addFarmDetailElements(mission.inGameMenu.pageSettingsGame) then
            Farm.resetToSingleplayer = Utils.overwrittenFunction(Farm.resetToSingleplayer, resetToSingleplayer)
            InGameMenuGameSettingsFrame.updateGameSettings = Utils.appendedFunction(InGameMenuGameSettingsFrame.updateGameSettings, updateGameSettings)
        end
    else
        g_logManager:info("Info: [SinglePlayerFarmDetails] Failed to load mod '%s', this mod serves no purpose in a Multiplayer game!")
    end
end

local function init()
    local mod = g_modManager:getModByName(modName)

    versionString = tostring(mod.version) or versionString

    if mod.modName == "FS19_SinglePlayerFarmDetails" or mod.modName == "FS19_SinglePlayerFarmDetails_update" and mod.author:len() == 3 then
        Mission00.load = Utils.prependedFunction(Mission00.load, loadMission)
    else
        loadError = {
            author = mod.author,
            startUpdateTime = 2000,

            update = function(self, dt)
                self.startUpdateTime = self.startUpdateTime - dt

                if self.startUpdateTime < 0 then
                    local text = string.format(g_i18n:getText("farmDetails_loadError", modName), modName, self.author)

                    if g_dedicatedServerInfo == nil then
                        if not g_gui:getIsGuiVisible() then
                            g_gui:showYesNoDialog({
                                title = string.format("%s - Version %s", mod.title, versionString),
                                text = text,
                                dialogType = DialogElement.TYPE_LOADING,
                                callback = self.openModHubLink,
                                target = nil,
                                yesText = g_i18n:getText("button_ok"),
                                noText = g_i18n:getText("button_modHubDownload")
                            })
                        end
                    else
                        print("\n" .. text .. "\n    - https://farming-simulator.com/mods.php?lang=en&country=be&title=fs2019&filter=org&org_id=129652&page=0" .. "\n")
                        self.openModHubLink(true)
                    end
                end
            end,

            openModHubLink = function(ignore)
                if ignore == false then
                    local language = g_languageShort
                    local link = "mods.php?lang=en&country=be&title=fs2019&filter=org&org_id=129652&page=0"

                    if language == "de" or language == "fr" then
                        link = "mods.php?lang=" .. language .. "&country=be&title=fs2019&filter=org&org_id=129652&page=0"
                    end

                    openWebFile(link, "")
                end

                removeModEventListener(loadError)
                loadError = nil
            end
        }

        addModEventListener(loadError)
    end
end

init()
